--
-- pmach.e
-- =======
--
-- Phix compatibility file for machine_func and machine_proc.
-- (Phix automatically includes this file /only/ if needed.)
--
-- The compiler converts eg allocate() to an opAlloc opcode; however 
-- old versions of machine.e and other legacy code might still invoke 
-- machine_func(16,n), so this maps it back.
--
-- Fairly obviously, this file is an unnecessary overhead for Phix, and 
-- there are 'machine_func deprecated' warnings to encourage getting rid.
-- That said, clearly the overhead of these wrappers on eg allocate must
-- be pretty insignificant, otherwise it would have gone long ago, and 
-- twice something pretty insignificant is still pretty insignificant.
--
--  <aside> Phix and RDS Eu use completely different back-ends, with a
--          completely different set of opcodes. There is no code at all
--          in the Phix backend for machine_func/proc, this is it.
--          This file exists purely for compatibility reasons.
--  </aside>
--
-- If you want to avoid warnings about machine_proc and machine_func being 
-- deprecated when running some existing code, just include pmach.e early 
-- on in the offending application, ideally after --/**/ so that RDS Eu
-- treats it as a comment but Phix does not, ie:
--
--      --/**/include pmach.e
--
-- Obviously as far as Phix is concerned, the better idea is to replace 
-- all machine_func(16,n) calls with allocate(n), etc. Phix ships with
-- versions of machine.e etc with special block comments, for example:
--
--      --/* Not required for Phix (defined as opAlloc)
--      global function allocate(positive_int n)
--          return machine_func(M_ALLOC, n)
--      end function
--      --*/
--
-- Phix treats everything between "--/*" and "--*/" as block comment,
-- whereas RDS Eu will process it normally. Also, if you see eg:
--
--  --/**/  if wait_key() then end if           --/*    -- Phix
--          if machine_func(26,0) then end if   --      -- RDS --*/
--
-- Then Phix treats "--/**/" as a block comment, calls wait_key(), and 
-- treats "--/* ... if machine_func(26,0) ... -- RDS --*/" as a comment, 
-- whereas RDS Eu treats the first line as a comment, calls machine_func(), 
-- and finally ignores the trailing "-- -- RDS --*/". (Ugly, but it works.)
--
-- Phix also honours "/*" & "*/" alone, but RDS Eu errors.
--   (btw, feel free to match your "--/*" with "*/" or "--*/".)
-- Note that "--<space>/*" is treated as a line comment.
-- Some old versions of win32lib contain a "--/*", which is immediately
--  flagged as an error (along with half a dozen implicit sequence ops
--  and a missing "(Phix compatible)" comment, all fixed in the version
--  of win32lib shipped with Phix.)
--
-- Clearly it is sensible, as shown, to label such (cross-platform) code with 
--  '--Phix' and '--RDS' hints, but these are not mandatory. Most such code
--  should, of course, be hidden in library routines/builtin includes.
--
-- The latest version of Edita handles Phix block comments; it shows the
-- last example with wait_key in the builtin colour and machine_func in the 
-- comment colour (ie Edita [0.2.8+] favours the Phix code).
-- Of course, the leading -- above make this whole block comment-coloured; 
-- block comments do not start inside line comments, though they can end 
-- inside them.
--

constant 
--  M_SOUND             = 1,    --sound()               DOS only (do nothing on win32)
--  M_LINE              = 2,    --draw_line()           DOS only
--  M_PALETTE           = 3,    --palette()             DOS only
--  ?                   = 4,
--  M_GRAPHICS_MODE     = 5,    --graphics_mode()       DOS only (do nothing on win32)
--  M_CURSOR            = 6,    --cursor()              DOS only (ish)[DEV]
--  M_WRAP              = 7,    --wrap()                DOS only (ish)[DEV]
--  M_SCROLL            = 8,    --scroll()              DOS only (ish)[DEV]
    M_SET_T_COLOR       = 9,    --text_color()
    M_SET_B_COLOR       = 10,   --bk_color()
--  M_POLYGON           = 11,   --polygon()             DOS only
--  M_TEXTROWS          = 12,   --text_rows()           DOS only (ish)[DEV]
    M_VIDEO_CONFIG      = 13,   --video_config()        DOS only
--  M_GET_MOUSE         = 14,   --get_mouse()           DOS only
--  M_MOUSE_EVENTS      = 15,   --mouse_events()        DOS only
    M_ALLOC             = 16,   --allocate()
    M_FREE              = 17,   --free()
--  M_ELLIPSE           = 18,   --ellipse()             DOS only
    M_SEEK              = 19,   --seek()
    M_WHERE             = 20,   --where()
--  ?                   = 21,
    M_DIR               = 22,   --dir()
    M_CURRENT_DIR       = 23,   --current_dir()
--  M_MOUSE_POINTER     = 24,   --mouse_pointer()       DOS only
    M_GET_POSITION      = 25,   --get_postion()
    M_WAIT_KEY          = 26,   --wait_key()
--  M_ALL_PALETTE       = 27,   --all_palette()         DOS only
--  M_GET_DISPLAY_PAGE  = 28,   --get_display_page()    DOS only
--  M_SET_DISPLAY_PAGE  = 29,   --set_display_page()    DOS only
--  M_GET_ACTIVE_PAGE   = 30,   --get_active_page()     DOS only
--  M_SET_ACTIVE_PAGE   = 31,   --set_active_page()     DOS only
--  M_ALLOC_LOW         = 32,   --allocate_low()        DOS only
--  M_FREE_LOW          = 33,   --free_low()            DOS only
--  M_INTERRUPT         = 34,   --dos_interrupt()       DOS only
    M_SET_RAND          = 35,   --set_rand()
--  M_USE_VESA          = 36,   --use_vesa()            DOS only
--  M_CRASH_MESSAGE     = 37,   --crash_message()               DEV
--  M_TICK_RATE         = 38,   --tick_rate()           DOS only
--  M_GET_VECTOR        = 39,   --get_vector()          DOS only
--  M_SET_VECTOR        = 40,   --set_vector()          DOS only
--  M_LOCK_MEMORY       = 41,   --lock_memory()         DOS only
--  M_ALLOW_BREAK       = 42,   --allow_break()                 DEV
--  M_CHECK_BREAK       = 43,   --check_break()                 DEV
--  ?                   = 44,
--  ?                   = 45,
    M_A_TO_F64          = 46,   --atom_to_float64()
    M_F64_TO_A          = 47,   --float64_to_atom()
    M_A_TO_F32          = 48,   --atom_to_float32()
    M_F32_TO_A          = 49,   --float32_to_atom()
    M_OPEN_DLL          = 50,   --open_dll()
    M_DEFINE_C          = 51,   --define_c_func(), define_c_proc()
    M_CALL_BACK         = 52,   --call_back()
--  -153,   --   ?      = 53,
    M_FREE_CONSOLE      = 54,   --free_console()
    M_INSTANCE          = 55,   --instance()
    M_DEFINE_VAR        = 56,   --define_c_var()
--  M_CRASH_FILE        = 57,   --crash_file()                  DEV
    M_GET_SCREEN_CHAR   = 58,   --get_screen_char()
    M_PUT_SCREEN_CHAR   = 59,   --put_screen_char()
    M_FLUSH             = 60,   --flush()
    M_LOCK_FILE         = 61,   --lock_file()
    M_UNLOCK_FILE       = 62,   --unlock_file()
    M_CHDIR             = 63,   --chdir()
    M_SLEEP             = 64    --sleep()

include pdir.e as pdir
include screen.e as screen
include cback.e as cback

global function machine_func(atom a, object x)
-- Compatibility layer, normally avoided, warnings issued when used (see above).
--
-- Assume: An old version of machine.e defines say allocate(n) as machine_func(M_ALLOC).
--         We cannot just code allocate(n) since it would cause an infinite loop.
--         Instead we hand-code the standard compiler binary output for such.
--         See pops.e for an introduction to #ilasm.
--
object r, p1, p2, p3, p4
    if a=M_ALLOC then
        #ilasm{ mov_edi_imm32,%isVar,0,0,r,             -- mov edi,result
                mov_esi_imm32,%isVar,0,0,x,             -- mov esi,size
                call_rel32,%isOpCode,0,0,%opAlloc}      -- [edi] = allocate([esi])
    elsif a=M_SEEK then
        p1 = x[1]   -- #ilasm cannot do things like subscripts
        p2 = x[2]   --  "
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                mov_edi_imm32,%isVar,0,0,p1,            -- mov edi,fileno
                mov_esi_imm32,%isVar,0,0,p2,            -- mov esi,seek value
                call_rel32,%isOpCode,0,0,%opSeek}       -- [edx] = seek([edi],[esi])
    elsif a=M_WHERE then
        #ilasm{ mov_ecx_imm32,%isVar,0,0,r,             -- mov ecx,result
                mov_eax_mem32,%isVar,0,0,x,             -- mov eax,[fileno]
                call_rel32,%isOpCode,0,0,%opWhere}      -- [ecx] = where(eax)
    elsif a=M_DIR then
        return pdir:dir(x)
    elsif a=M_CURRENT_DIR then
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                call_rel32,%isOpCode,0,0,%opCurrDir}    -- [edx] = current_dir()
    elsif a=M_GET_POSITION then
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                call_rel32,%isOpCode,0,0,%opGetPos}     -- [edx] = get_position()
    elsif a=M_WAIT_KEY then
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                call_rel32,%isOpCode,0,0,%opWaitKey}    -- [edx] = wait_key()
    elsif a=M_A_TO_F64 then
        #ilasm{ mov_edi_imm32,%isVar,0,0,r,             -- mov edi,result
                mov_edx_imm32,%isVar,0,0,x,             -- mov edx,x
                call_rel32,%isOpCode,0,0,%opAto64}      -- [edi] = atom_to_float64([edx])
    elsif a=M_F64_TO_A then
        #ilasm{ mov_ecx_imm32,%isVar,0,0,r,             -- mov ecx,result
                mov_eax_mem32,%isVar,0,0,x,             -- mov eax,[x]
                call_rel32,%isOpCode,0,0,%op64toA}      -- [ecx] = float64_to_atom(eax)
    elsif a=M_A_TO_F32 then
        #ilasm{ mov_edi_imm32,%isVar,0,0,r,             -- mov edi,result
                mov_edx_imm32,%isVar,0,0,x,             -- mov edx,x
                call_rel32,%isOpCode,0,0,%opAto32}      -- [edi] = atom_to_float32([edx])
    elsif a=M_F32_TO_A then
        #ilasm{ mov_ecx_imm32,%isVar,0,0,r,             -- mov ecx,result
                mov_eax_mem32,%isVar,0,0,x,             -- mov eax,[x]
                call_rel32,%isOpCode,0,0,%op32toA}      -- [ecx] = float32_to_atom(eax)
    elsif a=M_OPEN_DLL then
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                mov_eax_mem32,%isVar,0,0,x,             -- mov eax,[x]
                call_rel32,%isOpCode,0,0,%opOpenDLL}    -- [edx] = open_dll(eax)
    elsif a=M_DEFINE_C then
        p1 = x[1]   -- #ilasm cannot do things like subscripts
        p2 = x[2]   --  "
        p3 = x[3]   --  "
        p4 = x[4]   --  "
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                mov_edi_imm32,%isVar,0,0,p1,            -- mov edi,lib
                mov_esi_imm32,%isVar,0,0,p2,            -- mov esi,name
                mov_ecx_imm32,%isVar,0,0,p3,            -- mov ecx,params
                mov_eax_imm32,%isVar,0,0,p4,            -- mov eax,return type (0 for proc)
                call_rel32,%isOpCode,0,0,%opDefineC}    -- [edx] = define_c_func/proc([edi],[esi],[ecx],[eax])
    elsif a=M_CALL_BACK then
        return cback:call_back(x)
    elsif a=M_INSTANCE then
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                call_rel32,%isOpCode,0,0,%opInstance}   -- [edx] = instance()
    elsif a=M_DEFINE_VAR then
        p1 = x[1]   -- #ilasm cannot do things like subscripts
        p2 = x[2]   --  "
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                mov_edi_imm32,%isVar,0,0,p1,            -- mov edi,lib
                mov_esi_imm32,%isVar,0,0,p2,            -- mov esi,name
                call_rel32,%isOpCode,0,0,%opDefineV}    -- [edx] = define_c_var([edi],[esi])
    elsif a=M_LOCK_FILE then
        p1 = x[1]   -- #ilasm cannot do things like subscripts
        p2 = x[2]   --  "
        p3 = x[3]   --  "
        #ilasm{ mov_edx_imm32,%isVar,0,0,r,             -- mov edx,result
                mov_edi_imm32,%isVar,0,0,p1,            -- mov edi,fileno
                mov_ecx_imm32,%isVar,0,0,p2,            -- mov ecx,locktype
                mov_esi_imm32,%isVar,0,0,p3,            -- mov esi,byterange
                call_rel32,%isOpCode,0,0,%opLock}       -- [edx] = lock_file([edi],[ecx],[esi])
    elsif a=M_CHDIR then
        #ilasm{ mov_ecx_imm32,%isVar,0,0,r,             -- mov ecx,result
                mov_eax_mem32,%isVar,0,0,x,             -- mov eax,[x]
                call_rel32,%isOpCode,0,0,%opChDir}      -- [ecx] = chdir(eax)
    elsif a=M_VIDEO_CONFIG then
        return screen:video_config()
    elsif a=M_GET_SCREEN_CHAR then
        p1 = x[1]
        p2 = x[2]
        return screen:get_screen_char(p1,p2)
    else
        -- oops (please double check that you are using standard includes (eg file.e) 
        --       from Phix\builtins rather than those from Euphoria\include;
        --       actually they might/should work OK but please check anyway)
        printf(1,"machine_func %d %s\n",{a,sprint(x)})
        a = 9/0
    end if
    return r
end function

global procedure machine_proc(atom a, object x)
-- Compatibility layer, normally avoided, warnings issued when used (see above).
object p1, p2, p3
--  if a = M_SOUND then
--      -- do nothing[DEV]
--  elsif a = M_GRAPHICS_MODE then
--      -- do nothing
--  els
    if a=M_SET_T_COLOR then
        #ilasm{ mov_edx_imm32,%isVar,0,0,x,             -- mov edx,addr x
                call_rel32,%isOpCode,0,0,%opTxtClr}     -- text_color([edx])
    elsif a=M_SET_B_COLOR then
        #ilasm{ mov_edx_imm32,%isVar,0,0,x,             -- mov edx,addr x
                call_rel32,%isOpCode,0,0,%opBkClr}      -- bk_color([edx])
    elsif a=M_FREE then
        #ilasm{ mov_edi_imm32,%isVar,0,0,x,             -- mov edi,addr x
                call_rel32,%isOpCode,0,0,%opFree}       -- free([edi])
    elsif a=M_SET_RAND then
        #ilasm{ mov_edi_imm32,%isVar,0,0,x,             -- mov edi,addr x
                call_rel32,%isOpCode,0,0,%opSetRand}    -- set_rand([edi])
    elsif a=M_FREE_CONSOLE then
        #ilasm{ call_rel32,%isOpCode,0,0,%opFreeConsole}-- free_console()
    elsif a=M_FLUSH then
        #ilasm{ mov_edx_imm32,%isVar,0,0,x,             -- mov edx,addr x
                call_rel32,%isOpCode,0,0,%opFlush}      -- flush([edx])
    elsif a=M_UNLOCK_FILE then
        p1 = x[1]   -- #ilasm cannot do things like subscripts
        p2 = x[2]   --  "
        #ilasm{ mov_edi_imm32,%isVar,0,0,p1,            -- mov edi,fileno
                mov_esi_imm32,%isVar,0,0,p2,            -- mov esi,range
                call_rel32,%isOpCode,0,0,%opUnLock}     -- unlock_file([edi],[esi])
    elsif a=M_SLEEP then
        #ilasm{ mov_edi_imm32,%isVar,0,0,x,             -- mov edi,addr x
                call_rel32,%isOpCode,0,0,%opSleep}      -- sleep([edi])
    elsif a=M_PUT_SCREEN_CHAR then
        p1 = x[1]
        p2 = x[2]
        p3 = x[3]
        screen:put_screen_char(p1,p2,p3)
    else
        -- oops (please double check that you are using standard includes (eg file.e) 
        --       from Phix\builtins rather than those from Euphoria\include;
        --       actually they might/should work OK but please check anyway)
        printf(1,"machine_proc %d %s\n",{a,sprint(x)})
        a = 9/0
    end if
end procedure

